import java.util.LinkedList;
import java.util.PriorityQueue;
import java.util.Arrays;

/**
 * Die Klasse Dijkstra realisiert die
 * Suche des kuerzesten Weges nach Dijkstraa
 * 
 * @author claus albowski
 * @version Juni 2015
 */
public class Dijkstra
{
    private Graph graph;
    private Knoten start, ziel;
    private PriorityQueue<Weg> wege;

    /**
     * Konstruktor fuer Objekte der Klasse Dijkstraa
     */
    public Dijkstra(int anzahl, int breite, int hoehe)
    {
        graph = new Graph(anzahl, breite, hoehe);
        graph.zeichne();
        wege = new PriorityQueue();
    }

    /**
     * Konstruktor fuer Testzwecke
     */
    public Dijkstra()
    {
        graph = new Graph();
        graph.zeichne();
        wege = new PriorityQueue();
    }

    /**
     * set-Methode fuer den StartKnoten
     * "Knoten_1" ... "Knoten_<anzahl>"
     */
    public void setzeStartKnoten(String name)
    {
        start = Knoten.gibKnoten(name);
        start.setzeBewertung(0.0);
    }

    /**
     * set-Methode fuer den ZielKnoten
     * "Knoten_1" ... "Knoten_<anzahl>"
     */
    public void setzeZielKnoten(String name)
    {
        ziel = Knoten.gibKnoten(name);
    }

    /**
     * startet die Suche
     */
    public void starteSuche()
    {
        if (start==null) return;
        wege.add(new Weg(start));
        wege.peek().zeigeWeg();
    }

    /**
     * fuehrt die Suche fort
     */
    public void suche()
    {
        while (wege.peek().gibEndKnoten()!=ziel)
        expandiere();
        System.out.println("Loesung:");
        wege.peek().zeigeWeg();
        wege.peek().zeichneWeg();
    }

    /**
     * ein Expansionsschritt
     * der der Endknoten zm aktuell kuerzesten Weg wird expandiert
     * wenn er nicht bereits expandiert wurde
     */
    public void expandiere()
    {
        Weg kuerzester = wege.poll();
        Knoten aktuell = kuerzester.gibEndKnoten();
        // testen, ob der Endknoten schon expandiert ist
        if (!aktuell.istExpandiert())
        {
            aktuell.setzeExpandiert();
            System.out.println("expandiert: "+aktuell.gibNamen());
            PriorityQueue<Kante> nachfolgeKanten
            = aktuell.gibSortierteKanten();
            for (Kante neueKante : nachfolgeKanten)
            if (kuerzester.keinZyklus(neueKante))
            {
                Weg neuerWeg = new Weg(kuerzester, neueKante);
                if (neuerWeg.gibEndKnoten().gibBewertung()<0) // Endkoten noch unbewertet
                neuerWeg.gibEndKnoten().setzeBewertung(neuerWeg.wegLaenge()); // bewerten mit aktueller Laenge
                else
                if (neuerWeg.wegLaenge() < neuerWeg.gibEndKnoten().gibBewertung()) // neuer ist kuerzer
                neuerWeg.gibEndKnoten().setzeBewertung(neuerWeg.wegLaenge());
                neuerWeg.zeigeWeg();
                wege.add(neuerWeg);
            }
            zeigeAlleWegeGeordnet();
        }
    }

    public void zeigeAlleWegeGeordnet()
    {
        System.out.println("alle Wege geordnet: ");
        Weg[] alleWege = wege.toArray(new Weg[0]);
        Arrays.sort(alleWege);
        for (int i=0; i<alleWege.length; i++)
        alleWege[i].zeigeWeg();
    }
    
    ///// Interne Klasse Weg /////
    private class Weg
    implements Comparable
    {
        private LinkedList<Kante> kanten;
        private Knoten endKnoten;

        /**
         * Konstruktor fuer den Start
         */
        public Weg(Knoten startKnoten)
        {
            kanten = new LinkedList();
            endKnoten = startKnoten;
        }

        /**
         * Konstruktor fuer einen fortsetzenden Weg
         */
        public Weg(Weg weg, Kante neueKante)
        {
            kanten = new LinkedList();
            for (Kante kante : weg.gibKanten())
                kanten.addLast(kante);
            kanten.add(neueKante);
            endKnoten = neueKante.gibZielKnoten(); 
        }

        /**
         * get-Methode fuer die Kanten
         */
        public LinkedList<Kante> gibKanten() { return kanten; }

        /**
         * get-Methode fuer den Endknoten
         */
        public Knoten gibEndKnoten() { return endKnoten; }
        
        // Fuer das Interface  Comparable notwendige Methode:
        public int compareTo(Object obj) {
            if (wegLaenge()<((Weg)obj).wegLaenge()) return -1;
            else if (wegLaenge()>((Weg)obj).wegLaenge()) return 1;
            else return 0;
        }
        
        /**
         * prueft, ob eine Kante Knoten unter denen vor dem Endknoten enthaelt
         */
        public boolean keinZyklus(Kante neueKante)
        {
            // Ein Knoten der neuen Kante sollte der Endknoten des Weges sein
            Knoten frei = neueKante.gibZielKnoten();
            for (Kante kante : gibKanten())
            {
                if (frei==kante.gibAusgangsKnoten()) return false;
                if (frei==kante.gibZielKnoten()) return false;
            }
            return true;
        }
        
        /**
         * berechnet die Weglaenge
         */
        public double wegLaenge()
        {
            double laenge = 0;
            for (Kante kante : gibKanten()) laenge += kante.laenge();
            return laenge;
        }
        
        /**
         * Anzeige in der Konsole
         */
        public void zeigeWeg()
        {
            System.out.print(start.gibNamen());
            Knoten aktuell = start;
            for (Kante kante : kanten)
            System.out.print(" -> "+kante.gibZielKnoten().gibNamen());
            System.out.println(", Laenge "+wegLaenge());
        }
        
        /**
         * zeichnet den Weg rot ein
         */
        public void zeichneWeg()
        {
            for (Kante kante : gibKanten())
            Leinwand.gibLeinwand().zeichne(kante, "rot", kante);            
        }
    }

}
