import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.PriorityQueue;

/**
 * Die Klasse Knoten repraesentiert Knoten in einem Graphen.
 * Sie stellt auch Methoden zur Verwaltung aller Knoten bereit.
 * 
 * Erweiterung fuer Dijkstra um Bewertung
 * und um einen Namen zur Identifikation
 * 
 * @author claus albowski
 * @version Juni 2015
 */
public class Knoten extends Point2D.Double
implements Comparable
{
    // Klassenvariable
    private static ArrayList<Knoten> knotenListe = new ArrayList();
    private PriorityQueue<Knoten> bewerteteKnoten = new PriorityQueue();
    private static int x=300;
    private static int y=300;
    // Instanzvariable
    private PriorityQueue<Kante> kantenListe;
    private double bewertung;
    private boolean istExpandiert;
    private String name;
    
    /**
     * Konstruktor fuer zufaellig erzeugte Objekte der Klasse Knoten.
     */
    public Knoten() {
        super(x*Math.random(), y*Math.random());
        kantenListe = new PriorityQueue<Kante>();
        bewertung = -1.0;
        istExpandiert = false;
        knotenListe.add(this);
        name = "Knoten_"+knotenListe.size();
    }
    
    /**
     * Konstruktor fuer Objekte der Klasse Knoten mit vorgegebenen Werten.
     */
    public Knoten(double xPos, double yPos) {
        super(xPos, yPos);
        kantenListe = new PriorityQueue<Kante>();
        bewertung = -1.0;
        istExpandiert = false;
        knotenListe.add(this);
        name = "Knoten_"+knotenListe.size();
    }
    
    // Klassenmethoden
    /**
     * setzt die Dimensio fuer die Grafik
     */
    public static void setzeDimension(int breite, int hoehe){ x=breite;y=hoehe; }
    
    /**
     * gibt einen Knoten aus der Liste zurueck
     */
    public static Knoten gibKnoten(int i)
    {
        if (i<0) return null;
        if (!(i<knotenListe.size())) return null;
        return knotenListe.get(i);
    }
    
    /**
     * gibt zum Namen den Knoten zurueck
     */
    public static Knoten gibKnoten(String name)
    {
        return knotenListe.get(Integer.parseInt(name.substring(7))-1);
    }
    

    /**
     * set-Methode fuer die Bewertung
     */
    public void setzeBewertung(double bewertung)
    {
        this.bewertung = bewertung;
    }
    
    /**
     * get-Methode fuer die Bewertung
     */
    public double gibBewertung() { return bewertung; }
    
    /**
     * sondierende Methode fuer die Frage ob bewertet
     */
    public boolean istBewertet() { return !(bewertung<0.0); }
    
    // Fuer das Interface  Comparable notwendige Methode:
    public int compareTo(Object k) {
        if (bewertung<((Knoten)k).gibBewertung()) return -1;
        else if (bewertung>((Knoten)k).gibBewertung()) return 1;
        else return 0;
    }
    
    /**
     * setzt die Eigenschaft, expandiert zu sein
     */
    public void setzeExpandiert()
    {
        istExpandiert = true;
    }
    
    /**
     * get-Methode fuer die expandiert-Eigenschaft
     */
    public boolean istExpandiert()
    {
        return istExpandiert;
    }
    
    /**
     * get-Methode fuer den Namen
     */
    public String gibNamen() { return name; }
    
    /**
     * nimmt eine neue Kante auf.
     */
    public void neueKante(Kante k){
        kantenListe.add(k);
    }
    
    /**
     * entfernt eine vorhandene Kante.
     */
    public void entferneKante(Kante k){
        kantenListe.remove(k);
    }
    
    /**
     * gibt die von diesem Knoten ausgehenden Kanten sortiert zurueck.
     */
    public PriorityQueue<Kante> gibSortierteKanten(){
        return kantenListe;
    }
}
